#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import shlex
from pathlib import Path

from p1st.exec_print_chunk import execute_print_chunk
from p1st.repeat import repeat_until_successful


def main():
    args = parse_args()

    command_parts = (
        ['btrfs', 'send'],
        ['-p', str(args.parent)] if args.parent else [],
        ['--compress-data'] if args.compressed_data else [],
        [str(args.child)]
    )
    command = [x for xs in command_parts for x in xs]

    def get_chunk_path(chunk_no: int) -> Path:
        return args.chunk_dir.joinpath(f'{args.child.name}.CHUNK.{chunk_no}')

    def handle_chunk(chunk_no: int, last_chunk: bool):
        chunk_path = get_chunk_path(chunk_no)
        target_chunk_path = args.target_chunk_dir.joinpath(f'{args.child.name}.CHUNK.{chunk_no}')
        print(f'Handling chunk {chunk_path}')

        usr_confirmation_socket = chunk_path.parent.joinpath(f'{chunk_path.name}.SOCKET')
        target_socket = args.target_path.parent.joinpath(f'{args.target_path.name}.SOCKET')

        message = 'EOF' if last_chunk else 'OK'

        rsync_cmd = ['rsync',
                     str(chunk_path),
                     f'{args.ssh_target}:{str(target_chunk_path)}']
        inform_cmd = ['ssh',
                      args.ssh_target,
                      f'echo {message} | nc -U {shlex.quote(str(target_socket))}']

        repeat_until_successful(rsync_cmd, usr_confirmation_socket)
        repeat_until_successful(inform_cmd, usr_confirmation_socket)

    execute_print_chunk(
        command=command,
        get_chunk_path=get_chunk_path,
        handle_chunk=handle_chunk,
        chunk_size=args.chunk_size,
    )


def parse_args():
    parser = argparse.ArgumentParser(prog='btrfs-send-chunks')

    parser.add_argument('-p',
                        help='Path to parent subvolume; forwarded to btrfs-send.',
                        dest='parent',
                        default=None,
                        type=Path,
                        metavar='PARENT_SUBVOLUME'
                        )

    parser.add_argument('--compressed-data',
                        help='Forwarded to btrfs-send.',
                        dest='compressed_data',
                        action='store_true',
                        default=False,
                        )

    parser.add_argument('--chunk-size',
                        help='Size in bytes; defaults to 128 MB.',
                        dest='chunk_size',
                        type=int,
                        default=128 * 1024 * 1024,
                        )

    parser.add_argument('--chunk-dir',
                        help='Chunks are saved in this directory. '
                             'Defaults to parent directory of CHILD.',
                        dest='chunk_dir',
                        type=Path,
                        metavar='CHUNK_DIR',
                        default=None,
                        )

    parser.add_argument('--target-chunk-dir',
                        help='Chunks are saved in this directory on SSH_TARGET. '
                             'Must be an absolute path. '
                             'Defaults to parent directory of TARGET_PATH.',
                        dest='target_chunk_dir',
                        type=Path,
                        metavar='TARGET_CHUNK_DIR',
                        default=None,
                        )

    parser.add_argument('child',
                        help='Path to child subvolume. Forwarded to btrfs-send.',
                        type=Path,
                        metavar='CHILD_SUBVOLUME'
                        )

    parser.add_argument('ssh_target',
                        help='Hostname of target computer; as configured in ~/.ssh/config.',
                        metavar='SSH_TARGET'
                        )

    parser.add_argument('target_path',
                        help='Absolute path where the subvolume will be created on SSH_TARGET.',
                        type=Path,
                        metavar='TARGET_PATH'
                        )

    args = parser.parse_args()

    # Make all paths absolute. Set default values.
    args.child = args.child.absolute()
    if not args.target_path.is_absolute():
        raise ValueError('TARGET_PATH must be absolute')
    #
    if args.chunk_dir:
        args.chunk_dir = args.chunk_dir.absolute()
    else:  # Default value
        args.chunk_dir = args.child.absolute().parent
    if args.target_chunk_dir:
        if not args.target_chunk_dir.is_absolute():
            raise ValueError('TARGET_CHUNK_DIR must be absolute')
    else:  # Default value
        args.target_chunk_dir = args.target_path.parent
    #
    if args.parent:
        args.parent = args.parent.absolute()

    return args


if __name__ == '__main__':
    main()
